"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UploadImagesPlugin = void 0;
exports.findPlaceholder = findPlaceholder;
exports.handleImagePaste = exports.handleImageDrop = void 0;
var _state = require("@tiptap/pm/state");
var _view = require("@tiptap/pm/view");
const uploadKey = new _state.PluginKey("upload-image");
const UploadImagesPlugin = ({
  loadingClass,
  imageUploadFn
}) => new _state.Plugin({
  key: uploadKey,
  state: {
    init() {
      return _view.DecorationSet.empty;
    },
    apply(tr, set) {
      set = set.map(tr.mapping, tr.doc);
      const action = tr.getMeta(this);
      if (action?.add) {
        const {
          id,
          pos,
          src
        } = action.add;
        const placeholder = document.createElement("div");
        placeholder.setAttribute("class", "img-placeholder");
        const image = document.createElement("img");
        image.setAttribute("class", loadingClass);
        image.src = src;
        placeholder.appendChild(image);
        const deco = _view.Decoration.widget(pos, placeholder, {
          id
        });
        set = set.add(tr.doc, [deco]);
      } else if (action?.remove) {
        set = set.remove(set.find(void 0, void 0, spec => spec.id == action.remove.id));
      }
      return set;
    }
  },
  props: {
    decorations(state) {
      return this.getState(state);
    },
    handlePaste(view, event) {
      return handleImagePaste(view, event, imageUploadFn);
    },
    handleDrop(view, event) {
      return handleImageDrop(view, event, false, imageUploadFn);
    }
  }
});
exports.UploadImagesPlugin = UploadImagesPlugin;
const handleImagePaste = (view, event, uploadFn) => {
  if (event.clipboardData?.files.length) {
    event.preventDefault();
    const [file] = Array.from(event.clipboardData.files);
    if (!file) {
      return true;
    }
    const pos = view.state.selection.$from.parent.content.size === 0 ? view.state.selection.from - 1 : view.state.selection.from;
    if (uploadFn) {
      loadAndInsertRemoteImage(file, view, pos, uploadFn);
    } else {
      loadAndInsertLocalImage(file, view, pos);
    }
    return true;
  }
  return false;
};
exports.handleImagePaste = handleImagePaste;
const handleImageDrop = (view, event, moved, uploadFn) => {
  if (!moved && event.dataTransfer?.files.length) {
    event.preventDefault();
    const [file] = Array.from(event.dataTransfer.files);
    const coordinates = view.posAtCoords({
      left: event.clientX,
      top: event.clientY
    });
    if (!file) {
      return true;
    }
    const pos = coordinates?.pos ? coordinates.pos - 1 : 0;
    if (uploadFn) {
      loadAndInsertRemoteImage(file, view, pos, uploadFn);
    } else {
      loadAndInsertLocalImage(file, view, pos);
    }
    return true;
  }
  return false;
};
exports.handleImageDrop = handleImageDrop;
function findPlaceholder(state, id) {
  const decos = uploadKey.getState(state);
  const found = decos.find(void 0, void 0, spec => spec.id == id);
  return found.length ? found[0]?.from : null;
}
function loadImage(file) {
  return new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.readAsDataURL(file);
    reader.onload = () => {
      if (reader.result) {
        resolve(reader.result);
      } else {
        reject(new Error("Failed to load image"));
      }
    };
    reader.onerror = () => {
      reject(reader.error);
    };
  });
}
function insertImageNode(view, currentPos, pos, imageSrc, id) {
  const {
    schema,
    tr
  } = view.state;
  const node = schema.nodes.image?.create({
    src: imageSrc
  });
  if (!node) return;
  const transaction = tr.replaceWith(currentPos, pos, node).setMeta(uploadKey, {
    remove: {
      id
    }
  });
  view.dispatch(transaction);
}
const loadAndInsertLocalImage = async (file, view, pos) => {
  const id = {};
  const tr = view.state.tr;
  if (!tr.selection.empty) {
    tr.deleteSelection();
  }
  const src = await loadImage(file);
  if (!src) {
    return;
  }
  insertImageNode(view, pos, pos, src, id);
};
const loadAndInsertRemoteImage = async (file, view, pos, uploadFn) => {
  const id = {};
  const tr = view.state.tr;
  if (!tr.selection.empty) {
    tr.deleteSelection();
  }
  const placeholderSrc = await loadImage(file);
  if (!placeholderSrc) {
    console.error("Failed to load image");
    return;
  }
  insertPlaceholder(view, tr, pos, placeholderSrc, id);
  try {
    const src = await uploadFn(file);
    if (src === false) {
      const transaction = view.state.tr.delete(pos, pos).setMeta(uploadKey, {
        remove: {
          id
        }
      });
      view.dispatch(transaction);
      return;
    }
    const currentPos = findPlaceholder(view.state, id);
    if (currentPos === null) {
      const transaction = view.state.tr.delete(pos, pos).setMeta(uploadKey, {
        remove: {
          id
        }
      });
      view.dispatch(transaction);
      return;
    }
    insertImageNode(view, currentPos, currentPos, src, id);
  } catch (_error) {
    const transaction = view.state.tr.delete(pos, pos).setMeta(uploadKey, {
      remove: {
        id
      }
    });
    view.dispatch(transaction);
  }
};
async function insertPlaceholder(view, tr, pos, src, id) {
  tr.setMeta(uploadKey, {
    add: {
      id,
      pos,
      src
    }
  });
  view.dispatch(tr);
}